"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.createHookProvider = exports.createHookProviders = void 0;
const qs = require("querystring");
const rxjs_1 = require("rxjs");
const transform_1 = require("ng-packagr/lib/graph/transform");
const nodes_1 = require("ng-packagr/lib/ng-package/nodes");
const utils_1 = require("./utils");
const HOOK_HANDLERS = ['initTsConfig', 'analyseSources'];
class _TaskContext {
    constructor(_context, factoryInjections, graph) {
        this._context = _context;
        this.factoryInjections = factoryInjections;
        if (graph) {
            this.graph = graph;
        }
    }
    context() {
        return this._context;
    }
    taskArgs(key) {
        if (!this.parsedTaskArgs) {
            const { tasksArgs } = this.context().options;
            this.parsedTaskArgs = qs.parse(tasksArgs || '');
        }
        return this.parsedTaskArgs[key];
    }
}
function createHookProviders(hooksConfig, globalTasksContext) {
    const providers = [];
    const hookNames = Object.keys(utils_1.TRANSFORM_PROVIDER_MAP);
    for (const key of hookNames) {
        if (hooksConfig[key]) {
            providers.push(createHookProvider(key, hooksConfig[key], globalTasksContext));
        }
    }
    return providers;
}
exports.createHookProviders = createHookProviders;
function createHookProvider(sourceHookName, hookConfig, globalTasksContext) {
    const originalProvider = utils_1.TRANSFORM_PROVIDER_MAP[sourceHookName];
    if (!originalProvider) {
        throw new Error(`Invalid source hook name, ${sourceHookName} is not a recognized hook`);
    }
    const clonedProvider = Object.assign({}, originalProvider);
    clonedProvider.useFactory = (...args) => {
        const { before, replace, after } = hookConfig;
        const isEntryPointHandler = HOOK_HANDLERS.indexOf(sourceHookName) === -1;
        const taskContextFactory = graph => {
            const taskContext = new _TaskContext(globalTasksContext, args, graph);
            if (isEntryPointHandler) {
                taskContext.epNode = graph.find((0, nodes_1.isEntryPointInProgress)());
            }
            return taskContext;
        };
        const runners = [
            ...createHookTransform(before || [], taskContextFactory),
            ...createHookTransform(replace || [], taskContextFactory),
            !replace && originalProvider.useFactory(...args),
            ...createHookTransform(after || [], taskContextFactory),
        ].filter(t => !!t);
        return (0, rxjs_1.pipe)(...runners);
    };
    return clonedProvider;
}
exports.createHookProvider = createHookProvider;
function createHookTransform(tasksLike, taskContextFactory) {
    return tasksLike.map(handler => {
        return (0, transform_1.transformFromPromise)((graph) => __awaiter(this, void 0, void 0, function* () {
            return Promise.resolve(handler(taskContextFactory(graph))).then(g => g || graph);
        }));
    });
}
//# sourceMappingURL=create-hook-provider.js.map