"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.HookRegistry = void 0;
const utils_1 = require("./utils");
const job_1 = require("./job");
class HookRegistry {
    constructor(initialHooks) {
        this._jobs = [];
        this._hooks = {};
        if (initialHooks) {
            this._hooks = (0, utils_1.normalizeHooks)(initialHooks);
        }
    }
    get hasSelfWatchJob() {
        return this._jobs.some(job => job.internalWatch);
    }
    register(hookOrJob, handlers) {
        if (typeof hookOrJob === 'string') {
            const taskPhases = (0, utils_1.normalizeTaskPhases)(handlers);
            for (const phase of utils_1.HOOK_PHASES) {
                const handlers = taskPhases[phase];
                if (handlers) {
                    this.getHookPhase(hookOrJob, phase).push(...handlers);
                }
            }
        }
        else {
            const jobMeta = (0, job_1.findJobMetadata)(hookOrJob);
            if (!jobMeta) {
                throw new Error(`Unknown Job: ${hookOrJob}`);
            }
            this._jobs.push(jobMeta);
            const { hooks } = jobMeta;
            const hookNames = Object.keys(utils_1.TRANSFORM_PROVIDER_MAP);
            for (const key of hookNames) {
                if (hooks[key]) {
                    this.register(key, hooks[key]);
                }
            }
        }
        return this;
    }
    getHooks() {
        return this._hooks;
    }
    getJobs() {
        return this._jobs;
    }
    getHookPhase(hook, phase) {
        const hookMap = this._hooks[hook] || (this._hooks[hook] = {});
        if (!hookMap[phase]) {
            hookMap[phase] = [];
        }
        return hookMap[phase];
    }
}
exports.HookRegistry = HookRegistry;
//# sourceMappingURL=hook-registry.js.map