"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.createHooksContext = exports.ENTRY_POINT_STORAGE = exports.validateTypedTasks = exports.getTaskDataInput = exports.normalizeTaskPhases = exports.normalizeHooks = exports.HOOK_PHASES = exports.TRANSFORM_PROVIDER_MAP = void 0;
const rxjs_1 = require("rxjs");
const operators_1 = require("rxjs/operators");
const jsonc_parser_1 = require("jsonc-parser");
const core_1 = require("@angular-devkit/core");
const node_1 = require("@angular-devkit/core/node");
const init_tsconfig_di_1 = require("ng-packagr/lib/ng-package/entry-point/init-tsconfig.di");
const analyse_sources_di_1 = require("ng-packagr/lib/ng-package/entry-point/analyse-sources.di");
const entry_point_di_1 = require("ng-packagr/lib/ng-package/entry-point/entry-point.di");
const compile_ngc_di_1 = require("ng-packagr/lib/ng-package/entry-point/compile-ngc.di");
const write_bundles_di_1 = require("ng-packagr/lib/ng-package/entry-point/write-bundles.di");
const write_package_di_1 = require("ng-packagr/lib/ng-package/entry-point/write-package.di");
exports.TRANSFORM_PROVIDER_MAP = {
    initTsConfig: init_tsconfig_di_1.INIT_TS_CONFIG_TRANSFORM,
    analyseSources: analyse_sources_di_1.ANALYSE_SOURCES_TRANSFORM,
    entryPoint: entry_point_di_1.ENTRY_POINT_TRANSFORM,
    compileNgc: compile_ngc_di_1.COMPILE_NGC_TRANSFORM,
    writeBundles: write_bundles_di_1.WRITE_BUNDLES_TRANSFORM,
    writePackage: write_package_di_1.WRITE_PACKAGE_TRANSFORM
};
exports.HOOK_PHASES = ['before', 'replace', 'after'];
function normalizeHooks(hooks) {
    const hookNames = Object.keys(exports.TRANSFORM_PROVIDER_MAP);
    for (const key of hookNames) {
        if (hooks[key]) {
            hooks[key] = normalizeTaskPhases(hooks[key]);
        }
    }
    return hooks;
}
exports.normalizeHooks = normalizeHooks;
function normalizeTaskPhases(taskPhases) {
    for (const phase of exports.HOOK_PHASES) {
        const taskOrTasksLike = taskPhases[phase];
        if (taskOrTasksLike) {
            taskPhases[phase] = Array.isArray(taskOrTasksLike)
                ? taskOrTasksLike
                : [taskOrTasksLike];
        }
    }
    return taskPhases;
}
exports.normalizeTaskPhases = normalizeTaskPhases;
function getTaskDataInput(jobMeta, tasks) {
    const data = tasks.data || {};
    return { [jobMeta.selector]: data[jobMeta.selector] };
}
exports.getTaskDataInput = getTaskDataInput;
function validateTypedTasks(jobs, context) {
    return __awaiter(this, void 0, void 0, function* () {
        const tasks = context.options.tasks;
        const allHooksPromises = [];
        const promises = jobs.map(taskMeta => {
            return context.host.read((0, core_1.normalize)(taskMeta.schema))
                .pipe((0, operators_1.map)(buffer => core_1.virtualFs.fileBufferToString(buffer)), (0, operators_1.map)(str => (0, jsonc_parser_1.parse)(str, null, { allowTrailingComma: true })), (0, operators_1.switchMap)(schemaJson => {
                const contentJson = getTaskDataInput(taskMeta, tasks);
                // JSON validation modifies the content, so we validate a copy of it instead.
                const contentJsonCopy = JSON.parse(JSON.stringify(contentJson));
                return context.registry.compile(schemaJson)
                    .pipe((0, operators_1.concatMap)(validator => validator(contentJsonCopy)), (0, operators_1.concatMap)(validatorResult => {
                    return validatorResult.success
                        ? (0, rxjs_1.of)(contentJsonCopy)
                        : (0, rxjs_1.throwError)(new core_1.schema.SchemaValidationException(validatorResult.errors));
                }));
            }))
                .toPromise();
        });
        allHooksPromises.push(...promises);
        yield Promise.all(allHooksPromises);
    });
}
exports.validateTypedTasks = validateTypedTasks;
// Used to mimic the `data` object in `EntryPointNode` so we can bind things to an entry point through the pipes
// external to ng-packager.
exports.ENTRY_POINT_STORAGE = {
    ENTRY_POINT_DATA: new WeakMap(),
    get(node) {
        return this.ENTRY_POINT_DATA.get(node);
    },
    merge(node, data) {
        const currentData = this.ENTRY_POINT_DATA.get(node) || {};
        Object.assign(currentData, data);
        this.ENTRY_POINT_DATA.set(node, currentData);
    },
    delete(node) {
        return this.ENTRY_POINT_DATA.delete(node);
    }
};
function createHooksContext(options, context, host = new node_1.NodeJsSyncHost()) {
    var _a, _b;
    return __awaiter(this, void 0, void 0, function* () {
        const registry = new core_1.schema.CoreSchemaRegistry();
        registry.addPostTransform(core_1.schema.transforms.addUndefinedDefaults);
        const workspaceRoot = (0, core_1.normalize)(context.workspaceRoot);
        const { workspace } = yield core_1.workspaces.readWorkspace(workspaceRoot, core_1.workspaces.createWorkspaceHost(host));
        const projectName = (_b = (_a = context.target) === null || _a === void 0 ? void 0 : _a.project) !== null && _b !== void 0 ? _b : workspace.extensions['defaultProject'];
        if (!projectName) {
            throw new Error('Must either have a target from the context or a default project.');
        }
        const project = workspace.projects.get(projectName);
        const projectRoot = (0, core_1.resolve)(workspaceRoot, (0, core_1.normalize)(project.root));
        const projectSourceRoot = project.sourceRoot;
        const sourceRoot = projectSourceRoot
            ? (0, core_1.resolve)(workspaceRoot, (0, core_1.normalize)(projectSourceRoot))
            : undefined;
        return {
            logger: context.logger,
            root: workspaceRoot,
            projectRoot,
            sourceRoot,
            builderContext: context,
            options,
            host,
            registry,
        };
    });
}
exports.createHooksContext = createHooksContext;
//# sourceMappingURL=utils.js.map