"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Bump = void 0;
const operators_1 = require("rxjs/operators");
const Path = require("path");
const semver = require("semver");
const jsonc_parser_1 = require("jsonc-parser");
const core_1 = require("@angular-devkit/core");
const log = require("ng-packagr/lib/utils/log");
const build_1 = require("../build");
const VALID_BUMPS = [
    'major',
    'premajor',
    'minor',
    'preminor',
    'patch',
    'prepatch',
    'prerelease'
];
function bumpTask(context) {
    return __awaiter(this, void 0, void 0, function* () {
        const bump = context.taskArgs('bump');
        if (!bump || context.epNode.data.entryPoint.isSecondaryEntryPoint) {
            return;
        }
        if (VALID_BUMPS.indexOf(bump) === -1) {
            const err = new Error(`BumpTask: Invalid semver version bump, ${bump} is not a known semver release type`);
            log.error(err.message);
            throw err;
        }
        const { entryPoint } = context.epNode.data;
        const ver = semver.parse(entryPoint.packageJson.version);
        const oldVersion = ver.version;
        const newVersion = semver.inc(ver, bump);
        entryPoint.packageJson.version = newVersion;
        const { host } = context.context();
        const packageJsonPath = (0, core_1.normalize)(Path.join(entryPoint.basePath, 'package.json'));
        yield host.read(packageJsonPath)
            .pipe((0, operators_1.map)(buffer => core_1.virtualFs.fileBufferToString(buffer)), (0, operators_1.map)(str => (0, jsonc_parser_1.parse)(str, null, { allowTrailingComma: true })), (0, operators_1.tap)(packageJson => packageJson.version = newVersion), (0, operators_1.switchMap)(packageJson => {
            return host.write(packageJsonPath, core_1.virtualFs.stringToFileBuffer(JSON.stringify(packageJson, null, 2)));
        }))
            .toPromise();
        log.msg(`Version bumped from ${oldVersion} to ${newVersion} (${bump})`);
    });
}
let Bump = class Bump {
};
Bump = __decorate([
    (0, build_1.Job)({
        schema: Path.resolve(__dirname, 'bump.json'),
        selector: 'bump',
        hooks: {
            writePackage: {
                before: bumpTask
            }
        }
    })
], Bump);
exports.Bump = Bump;
//# sourceMappingURL=bump.js.map