"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.CopyFile = void 0;
const Path = require("path");
const FS = require("fs");
const globby = require("globby");
const normalize_asset_patterns_1 = require("@angular-devkit/build-angular/src/utils/normalize-asset-patterns");
const log = require("ng-packagr/lib/utils/log");
const build_1 = require("../build");
function buildCopyPatterns(root, assets) {
    return assets.map(asset => {
        var _a;
        // Resolve input paths relative to workspace root and add slash at the end.
        asset.input = Path.resolve(root, asset.input).replace(/\\/g, '/');
        asset.input = asset.input.endsWith('/') ? asset.input : asset.input + '/';
        asset.output = asset.output.endsWith('/') || ((_a = asset.explicitFileName) === null || _a === void 0 ? void 0 : _a.length) > 0 ? asset.output : asset.output + '/';
        if (asset.output.startsWith('..')) {
            const message = 'An asset cannot be written to a location outside of the output path.';
            throw new Error(message);
        }
        return {
            context: asset.input,
            // Now we remove starting slash to make Webpack place it from the output root.
            to: asset.output.replace(/^\//, ''),
            ignore: asset.ignore,
            explicitFileName: asset.explicitFileName,
            from: {
                glob: asset.glob,
                dot: true,
            },
        };
    });
}
function createCopyPatterns(assetPatterns, root, projectRoot, maybeSourceRoot) {
    const assets = (0, normalize_asset_patterns_1.normalizeAssetPatterns)(assetPatterns.map(p => {
        var _a;
        if (!(typeof p == "string" || p instanceof String) && ((((_a = p.input) === null || _a === void 0 ? void 0 : _a.length) || 0) == 0)) {
            p.input = projectRoot;
        }
        return p;
    }), root, projectRoot, maybeSourceRoot);
    return buildCopyPatterns(root, assets);
}
function getGlobEntries(copyPattern, copyOptions) {
    return __awaiter(this, void 0, void 0, function* () {
        const fullPattern = copyPattern.context + copyPattern.from.glob;
        const opts = Object.assign(Object.assign({}, copyOptions), { dot: copyPattern.from.dot });
        return globby(fullPattern, opts);
    });
}
function executeCopyPattern(copyPattern, copyOptions, root, onCopy) {
    var _a;
    return __awaiter(this, void 0, void 0, function* () {
        const entries = yield getGlobEntries(copyPattern, copyOptions);
        if (((_a = copyPattern.explicitFileName) === null || _a === void 0 ? void 0 : _a.length) > 0 && entries.length > 1) {
            throw new Error(`Using 'explicitFileName' requires the glob to resolve to a single file. [Input]: ${copyPattern.context}`);
        }
        for (const entry of entries) {
            const cleanFilePath = entry.replace(copyPattern.context, '');
            const to = Path.resolve(root, copyPattern.to, copyPattern.explicitFileName || cleanFilePath);
            const pathToFolder = Path.dirname(to);
            pathToFolder.split('/').reduce((p, folder) => {
                p += folder + '/';
                if (!FS.existsSync(p)) {
                    FS.mkdirSync(p);
                }
                return p;
            }, '');
            FS.copyFileSync(entry, to);
            if (onCopy) {
                onCopy(entry, to);
            }
        }
    });
}
function executeCopyPatterns(copyPatterns, root, copyOptions, onCopy) {
    return __awaiter(this, void 0, void 0, function* () {
        const opts = copyOptions ? Object.assign({}, copyOptions) : {};
        for (const copyPattern of copyPatterns) {
            const singleOnCopy = onCopy
                ? (from, to) => onCopy(copyPattern, from, to)
                : undefined;
            yield executeCopyPattern(copyPattern, opts, root, singleOnCopy);
        }
    });
}
function copyFilesTask(context) {
    return __awaiter(this, void 0, void 0, function* () {
        const globalContext = context.context();
        if (context.epNode.data.entryPoint.isSecondaryEntryPoint) {
            return;
        }
        const { builderContext, options, root } = globalContext;
        const copyPatterns = createCopyPatterns(options.tasks.data.copyFile.assets, root, globalContext.projectRoot, globalContext.sourceRoot);
        const copyOptions = { ignore: ['.gitkeep', '**/.DS_Store', '**/Thumbs.db'] };
        const onCopy = (pattern, from, to) => {
            log.success(` - from: ${from}`);
            log.success(` - to: ${to}`);
        };
        log.info('Copying assets');
        try {
            yield executeCopyPatterns(copyPatterns, root, copyOptions, onCopy);
        }
        catch (err) {
            builderContext.logger.error(err.toString());
            throw err;
        }
    });
}
let CopyFile = class CopyFile {
};
CopyFile.copyFilesTask = copyFilesTask;
CopyFile.createCopyPatterns = createCopyPatterns;
CopyFile.executeCopyPattern = executeCopyPattern;
CopyFile.executeCopyPatterns = executeCopyPatterns;
CopyFile = __decorate([
    (0, build_1.Job)({
        schema: Path.resolve(__dirname, 'copy-file.json'),
        selector: 'copyFile',
        hooks: {
            writePackage: {
                before: copyFilesTask
            }
        }
    })
], CopyFile);
exports.CopyFile = CopyFile;
//# sourceMappingURL=copy-file.js.map