import * as ts from 'typescript';
import { TaskContext, EntryPointTaskContext } from '../build';
declare module '../build/hooks' {
    interface NgPackagrBuilderTaskSchema {
        nodeLib: {
            /**
             * The file path of the TypeScript configuration file, specific for node library generation.
             * If not set will use the tsConfig of the project
             */
            tsConfig?: string;
            /**
             * Compiler options overriding the one's in the file
             */
            compilerOptions?: ts.CompilerOptions;
        };
    }
}
declare module '../build/utils' {
    interface EntryPointStorage {
        nodeLib: {
            tsConfig: Pick<ts.CreateProgramOptions, 'rootNames' | 'options' | 'projectReferences'>;
            watchProgram?: ts.WatchOfFilesAndCompilerOptions<ts.EmitAndSemanticDiagnosticsBuilderProgram>;
        };
    }
}
/**
 * Run after the original initTsConfig and load the user tsconfig override, if given or the root tsConfig file.
 * We reload the configuration from scratch and create configuration parameters for a simple tsc compilation without the angular compiler stuff.
 * We run "after" and not "replace" because there are a lot of areas depending on the angular compiler `ParsedConfiguration` object.
 */
declare function initTsConfig(context: TaskContext<[import('@angular/compiler-cli').ParsedConfiguration]>): Promise<import("ng-packagr/lib/graph/build-graph").BuildGraph>;
/**
 * Replace the original compilation step, compile for node and not for angular.
 */
declare function compilerNgc(context: EntryPointTaskContext): Promise<void>;
/**
 * Replace the original package.json create logic, write values for a node library.
 */
declare function writePackage(context: EntryPointTaskContext): Promise<void>;
export declare class NodeLib {
    static readonly initTsConfig: typeof initTsConfig;
    static readonly compilerNgc: typeof compilerNgc;
    static readonly writePackage: typeof writePackage;
}
export {};
//# sourceMappingURL=node-lib.d.ts.map